﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web.Mvc;
using igGridSample.Models;
using Infragistics.Web.Mvc;

namespace igGridSample.Controllers
{
    public class HomeController : Controller
    {
        //
        // GET: /Home/
        private List<Product> myList = GetProducts();

        public ActionResult Index()
        {
            return RedirectToAction("Grid");
        }

        public ActionResult Grid()
        {
            var gridModel = GetGridModel("test", 0);
            gridModel.LoadOnDemand = true;
            return View(gridModel);
        }

        private GridModel GetGridModel(string ajaxUpdateTarget, int totalRecordCount)
        {
            GridModel gridModel = new GridModel();
            gridModel.DataSourceUrl = string.Format("/Home/GetParent?ajaxUpdateTarget={0}", ajaxUpdateTarget);
            gridModel.ID = string.Format("grid-id-{0}", ajaxUpdateTarget);
            gridModel.AutoGenerateColumns = false;
            gridModel.AutoGenerateLayouts = false;
            gridModel.AutofitLastColumn = true;
            gridModel.Width = "100%";

            gridModel.Features.Add(new GridFiltering() { Type = OpType.Local, Inherit = false });
            gridModel.Features.Add(new GridSorting() { Type = OpType.Remote, Inherit = false });
            //gridModel.Features.Add(new GridSummaries() { Type = OpType.Local, Inherit = true });
            gridModel.Features.Add(new GridPaging() { Type = OpType.Remote, Inherit = false, PageSize = 10 });
            gridModel.Features.Add(new GridResizing() { Inherit = true });
            gridModel.Features.Add(new GridSelection() { Mode = SelectionMode.Row, MultipleSelection = false });
            gridModel.Features.Add(new GridHiding() { Inherit = true});
            gridModel.Features.Add(new GridColumnMoving() { Inherit = false, AddMovingDropdown = true, MoveType = MovingType.Dom });
            gridModel.Features.Add(new GridTooltips() { Inherit = true, Visibility = TooltipsVisibility.Always });
            //gridModel.Features.Add(new GridAppendRowsOnDemand() { LoadTrigger = LoadTrigger.Auto, Type = OpType.Remote, ChunkSize = 10, TotalRecordsCount = totalRecordCount });


            gridModel.PrimaryKey = "ProductID";
            gridModel.Columns.Add(new GridColumn() { HeaderText = "Name", Key = "Name", DataType = "string" });
            gridModel.Columns.Add(new GridColumn() { HeaderText = "Product Number", Key = "ProductNumber", DataType = "string" });


            GridColumnLayoutModel layout = new GridColumnLayoutModel();
            layout.AutoGenerateColumns = false;
            layout.Key = "Histories";
           // layout.ForeignKey = "ProductID";
            layout.Columns.Add(new GridColumn() { HeaderText = "Something", Key = "Something", DataType = "string" });
           // layout.Columns.Add(new GridColumn() { HeaderText = "ProductID", Key = "ProductID", DataType = "number" });
            layout.LoadOnDemand = false;
            //layout.Features.Add(new GridHiding() { Inherit = true, ColumnSettings = new List<ColumnHidingSetting>() { 
            //new ColumnHidingSetting(){ColumnKey = "ProductID",AllowHiding= false, Hidden=true }}
            //});


            layout.DataSourceUrl = string.Format("/Home/GetChildren?ajaxUpdateTarget={0}", "History");
            layout.Width = "100%";
            gridModel.ColumnLayouts.Add(layout);



            //layout = new GridColumnLayoutModel();
            //layout.Key = "Extras";
            //layout.AutoGenerateColumns = false;
            //layout.Columns.Add(new GridColumn() { HeaderText = "SomethingNew", Key = "SomethingNew", DataType = "string" });
            //layout.Columns.Add(new GridColumn() { HeaderText = "ProductNewID", Key = "SomethingNew", DataType = "string" });
            //layout.LoadOnDemand = false;
            //layout.ResponseDataKey = "Records";
            //layout.DataSourceUrl = string.Format("/Home/GetChildren?ajaxUpdateTarget={0}", "SomethingNew");
            //layout.Width = "100%";
            //gridModel.ColumnLayouts.Add(layout);

            return gridModel;
        }

        public JsonResult GetParent(string ajaxUpdateTarget, int page = 0, int pageSize = 10)
        {
            // get the records count from SQL server
            int totalRecords = 1000;
            // our database is Millions of record. We only show 10 or 100 a time,
            // The reason I use Json here because The TotalRecordsCount will be computer on the SQL Server Side
            return this.Json(new
            {
                Page = page,
                PageSize = pageSize,
                Records = myList.Skip(pageSize * page).Take(pageSize).ToArray(),
                TotalRecordsCount = totalRecords,
            }, JsonRequestBehavior.AllowGet);
        }

        public JsonResult GetChildren(string ajaxUpdateTarget, string path, string layout)
        {
            if (ajaxUpdateTarget == "History")
            {
                var histories = new List<ProductHistory>();
                histories.Add(new ProductHistory() { Something = "What" });
                GridModel gridModel = this.GetGridModel(ajaxUpdateTarget, 0);

                //gridModel.DataSource = histories.AsQueryable();
                WrappedGridResponse response;
                response = new WrappedGridResponse(histories, null);
                return Json(response, JsonRequestBehavior.AllowGet);
               // return gridModel.GetData(path, layout);
            }
            else
            {

                var something = new List<ProductExtraSomething>();

                GridModel gridModel = this.GetGridModel(ajaxUpdateTarget, 0);

                gridModel.DataSource = something.AsQueryable();
                return gridModel.GetData();

                //return this.Json(new
                //{
                //    Record = resultSet.Result
                //}, JsonRequestBehavior.AllowGet);
            }
        }

        static List<Product> GetProducts()
        {
            List<Product> products = new List<Product>();

            for (int i = 0; i < 1000; i++)
            {
                products.Add(new Product() { ProductID = i, Name = "Adjustable Race " + i.ToString(), ProductNumber = "AR-5381", ReleaseDate = DateTime.Now.AddDays(i) });
            }

            return products;
        }

        public JsonResult Show(string id, int page = 0, int pageSize = 100, DateTime? minDate = null, DateTime? maxDate = null)
        {
            var model = new ShowModel();
            model.Id = id;
            model.TotalRecordsCount = myList.Count;
            model.Entries = myList.Skip(pageSize * page).Take(pageSize).ToArray();
            return Json(model, JsonRequestBehavior.AllowGet);
        }
    }
}
